# -*- coding: utf-8 -*-

# Project Library
from kinder.core.children.models import Delegate, Children

from concentrator.models import DelegatePerson
from concentrator.rules import *
from concentrator.webservice.django_objects_proxy import (
    DelegateProxy, ChildrenProxy)
from concentrator.webservice.helpers import get_delegate

from lipetsk_specifics.models import (
    PrivilegeConfirmationAttributes, PrivilegeOwnerEnum)


class LipetskDelegateProxy(DelegateProxy):
    """
    добавили сохранение AddressResidence AddressRegistration
    """

    def get(self):
        delegate = Delegate(
            firstname=self._delegate.FirstName,
            surname=self._delegate.LastName,
            patronymic=self._delegate.MiddleName,
            snils=self._delegate.Snils,
            dul_type_id=self._get_doc_type(),
            dul_series=self._delegate.DocSeria,
            dul_number=self._delegate.DocNumber,
            dul_date=self._delegate.DocIssueDate,
            dul_place=self._delegate.DocIssuerDepartmentCode,
            dul_issued_by=self._delegate.DocIssuerName,
            type=self._get_delegate_type(),
            email=self._delegate.Email,
            phones=self._delegate.PhoneNumber,
            address_full=self._delegate.AddressResidence,
            reg_address_full=self._delegate.AddressRegistration)

        delegate_extension = DelegatePerson(delegate=delegate,
                                            doc_type=int(self._delegate.DocType)
        )
        return delegate, delegate_extension


class DelegatePrivilegeProxy(DelegateProxy):
    """
    Прокси класс для работы с обладателем льготы
    """

    def get(self):
        delegate = Delegate(
            firstname=self._delegate.FirstName,
            surname=self._delegate.LastName,
            patronymic=self._delegate.MiddleName,
            snils=self._delegate.Snils,
            dul_type_id=self._get_doc_type(),
            dul_series=self._delegate.DocSeria,
            dul_number=self._delegate.DocNumber,
            dul_date=self._delegate.DocIssueDate,
            dul_issued_by=self._delegate.DocIssuerName,
            type=self._delegate.Type)

        delegate_extension = DelegatePerson(delegate=delegate,
                                            doc_type=int(self._delegate.DocType)
        )
        return delegate, delegate_extension


class LipetskChildrenProxy(ChildrenProxy):
    """
        добавили сохранение DateOfActNumber
    """

    def get(self):
        return Children(
            firstname=self._children.FirstName,
            surname=self._children.LastName,
            patronymic=self._children.MiddleName,
            snils=self._children.Snils,
            address_full=self._children.AddressResidence,
            reg_address_full=self._children.AddressRegistration,
            dul_type=DULTypeEnumerate.SVID,
            dul_series=self._children.BirthDocSeria,
            dul_number=self._children.BirthDocNumber,
            dul_date=self._children.BirthDocIssueDate,
            gender=self._get_sex(),
            date_of_birth=self._children.DateOfBirth,
            health_need=self._get_health_need(),
            zags_act_number=self._children.BirthDocActNumber,
            zags_act_place=self._children.BirthDocIssuer,
            birthplace=self._children.BirthPlace,
            zags_act_date=self._children.DateOfActNumber)


class PrivConfirmAttrsProxy(object):
    u"""Прокси класс для работы с доп.аттрибутами льготы (фича Липецка)."""

    def __init__(self, child_data, declaration, declaration_privilege):
        """
        :param child_data: spyne-модель информации о ребенке
        :type child_data: DeclaredPersonData
        :param declaration: заявление django-модель)
        :type declaration: Declaration
        :param declaration_privilege: льгота по заявке (django-модель)
        :type declaration_privilege: DeclarationPrivilege
        """
        self._child_data = child_data
        self.declaration = declaration
        self._delegate = get_delegate(declaration)
        self._declaration_privilege = declaration_privilege
        self.privilege_owner = self._get_privilege_owner()

    def _get_privilege_owner(self):
        u"""Получение значения обладателя льготы из spyne-модели."""
        if self._child_data.Benefits:
            return self._child_data.Benefits.WhoHaveBenefit.Type
        else:
            return None

    def get(self):
        """Данные в теге  WhoHaveBenefit, помимо поля Type,
        приходят только для владельца льгот "Представитель"
        """
        if self.privilege_owner == PrivilegeOwnerEnum.DELEGATE:
            new_delegate, new_delegate_extension = DelegatePrivilegeProxy(
                self._child_data.Benefits.WhoHaveBenefit).get()
            return PrivilegeConfirmationAttributes(
                delegate=new_delegate,
                declaration_privilege=self._declaration_privilege,
                privilege_owner=self.privilege_owner,
                portal=True
            )
        else:
            return PrivilegeConfirmationAttributes(
                declaration_privilege=self._declaration_privilege,
                privilege_owner=self.privilege_owner,
                portal=True
            )