# coding:utf-8
"""
Created on 14.11.14

@author: kasimova
"""
from django.db.models import Q

from kinder.webservice.spyne_ws.declaration_info.logics import (
    CheckDeclarationProcess)
from lipetsk_specifics.models import (
    PrivilegeConfirmationAttributes)
from .descriptors import SummaryWithOutAgePosition


def get_decl_priv(declaration, for_change=False):
    """
    Возвращает наиболее приоритетную льготу из заявки
    Для метода изменения пришедший список сравниваем совсем списком льгот
    @param declaration: заявление
    @return: list of DeclsrationPrivilege или []

    """
    benefits = []
    if for_change:
        benefits = list(declaration.declarationprivilege_set.all())
        return benefits
    else:
        confir_attr = get_priv_data(declaration)
        if confir_attr:
            benefits = [confir_attr.declaration_privilege, ]
    return benefits


def get_priv_data(declaration):
    """
    возвращает информацию о подтверждении приоритетной льготы, если она есть
    @param declaration: Заявление
    @return: DeclarationPrivilege или None
    """
    priv_conf_attr = None
    best_privilege = declaration.best_privilege
    if best_privilege:
        privilege_filter = Q(
            declaration_privilege__privilege_id=best_privilege.id,
            declaration_privilege__declaration_id=declaration.id)
        try:
            # Выбираем доп информацию среди подтвержденных данных
            # по данной льготе для данного заяления
            priv_conf_attr = PrivilegeConfirmationAttributes.objects.get(
                privilege_filter)
        except PrivilegeConfirmationAttributes.DoesNotExist:
            # Если в заявке нет "лучшей" льготы, то и других льгот тоже нет
            pass
    return priv_conf_attr


class RegionalCheckDeclarationProcess(CheckDeclarationProcess):
    """Расширяем базовый класс

    Включаем несколько новых тегов в ответ. Изменяем формат даты.
    """

    summary_with_out_age = SummaryWithOutAgePosition()

    def _get_base_unit_queue_dict(self, declaration_unit):

        unit_queue = super(
            RegionalCheckDeclarationProcess, self
        )._get_base_unit_queue_dict(
            declaration_unit)

        unit_queue.update(DeclID=self.declaration.client_id)

        #Заменяем значения дат строковыми представлениями
        unit_queue['Date'] = unit_queue['Date'].strftime(
            '%d.%m.%Y %H:%m:%S')
        unit_queue['DesiredDate'] = unit_queue['DesiredDate'].strftime(
            '%d.%m.%Y') if unit_queue['DesiredDate'] else ''

        return unit_queue

    def _get_indexes(self, declaration_unit):

        result_dict = super(
            RegionalCheckDeclarationProcess,
            self)._get_indexes(declaration_unit)

        result_dict.update(
            AllCategoryPosition=self.summary_with_out_age
        )

        return result_dict
